==> jena-4.1.0-source-release.zip: jena-4.1.0/jena-base/src/main/java/org/apache/jena/atlas/iterator/PushbackIterator.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jena.atlas.iterator ;

import java.util.ArrayDeque ;
import java.util.Deque ;
import java.util.Iterator ;

/** An iterator where you can push items backinto the iterator, to be yielded (LIFO) next time.
 *  @see PeekIterator
 */
public class PushbackIterator<T> implements Iterator<T> {
    private Deque<T>    items = new ArrayDeque<>() ;
    private Iterator<T> iter ;

    public PushbackIterator(Iterator<T> iter) {
        if ( iter == null )
            throw new IllegalArgumentException("Wrapped iterator can't be null") ;
        this.iter = iter ;
    }

    public void pushback(T item) {
        items.push(item) ;
    }

    @Override
    public boolean hasNext() {
        if ( !items.isEmpty() )
            return true ;
        return iter.hasNext() ;
    }

    @Override
    public T next() {
        if ( !items.isEmpty() )
            return items.pop() ;
        return iter.next() ;
    }
}

==> jena-5.6.0-source-release.zip: jena-5.6.0/jena-base/src/main/java/org/apache/jena/atlas/iterator/PushbackIterator.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jena.atlas.iterator ;

import java.util.ArrayDeque ;
import java.util.Deque ;
import java.util.Iterator ;
import java.util.function.Consumer;

/**
 * An iterator where you can push items back into the iterator, to be yielded (LIFO) next time.
 * @see PeekIterator
 */
public class PushbackIterator<T> implements Iterator<T> {
    private Deque<T>    items = new ArrayDeque<>() ;
    private Iterator<T> iter ;

    public PushbackIterator(Iterator<T> iter) {
        if ( iter == null )
            throw new IllegalArgumentException("Wrapped iterator can't be null") ;
        this.iter = iter ;
    }

    public void pushback(T item) {
        items.push(item) ;
    }

    @Override
    public boolean hasNext() {
        if ( !items.isEmpty() )
            return true ;
        return iter.hasNext() ;
    }

    @Override
    public T next() {
        if ( !items.isEmpty() )
            return items.pop() ;
        return iter.next() ;
    }

    @Override
    public void forEachRemaining(Consumer<? super T> action) {
        while(!items.isEmpty()) {
            action.accept(items.pop());
        }
        iter.forEachRemaining(action);
    }
}
